package net.zenrindatacom.locationservice.sample.activity;

import android.os.Bundle;
import android.util.Log;

import androidx.appcompat.app.AppCompatActivity;
import androidx.preference.Preference;

import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.preference.AppPreference;

/**
 * アプリ設定画面
 *
 * @author N.Miyahara
 */
public class AppSettingActivity extends AppCompatActivity {
    /**
     * onCreate
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getSupportFragmentManager().beginTransaction().replace(android.R.id.content, new AppSettingPreferenceFragment()).commit();
    }

    /**
     * 本画面のPreferenceFragmentクラス
     */
    public static class AppSettingPreferenceFragment extends SettingPreferenceFragment implements Preference.OnPreferenceClickListener {
        /**
         * 親クラスのonCreatePreferences()イベント発生時に呼び出される
         * 　本タイミングで初期化に必要な処理を行う。
         */
        @Override
        public void onCreateSettingPreference(String rootKey) {
            //対象とするプリファレンスファイル名をデフォルトから変更
            getPreferenceManager().setSharedPreferencesName(AppPreference.getInstance().getPreferenceName());
            //本設定画面と対になるPreferenceScreen設定ファイル
            setPreferencesFromResource(R.xml.app_setting, rootKey);
            //タップ時のリスナー登録（「デフォルト設定に戻す」ボタン）
            Preference preference = findPreference(getString(R.string.prf_app_key_return_to_default_setting));
            if(preference != null) {
                preference.setOnPreferenceClickListener(this);
            }
        }

        /**
         * PreferenceClickリスナー
         * 　PreferenceScreen上のボタンがタップされるとCALLされる
         *
         * @return イベントハンドリング結果
         */
        @Override
        public boolean onPreferenceClick(Preference preference) {

            Log.i("debug-TAG", "Tap!! (" + preference.getKey() + ")");

            if(preference.getKey().equals(getString(R.string.prf_app_key_return_to_default_setting))) {
                //「デフォルト設定に戻す」ボタンをタップ
                AppPreference.getInstance().returnToDefault();
                return true;
            }

            //処理なし
            return false;
        }
    }
}
