package net.zenrindatacom.locationservice.sample.locationlib;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;

import net.zenrindatacom.locationservice.sample.application.SampleApplication;
import net.zenrindatacom.locationservice.sample.R;
import net.zenrindatacom.locationservice.sample.preference.LocationLibPreference;

import java.util.ArrayList;

/**
 * 位置測位ライブラリへ引き渡すIntentの情報を設定するクラス
 *
 * @author K.Hatano
 */
public class LocationLibSetting {

    /**
     * 引数のIntentに「測位開始」パラメータを設定する
     *
     * @param intent パラメータ設定先のIntent
     */
    public static void setIntentStartValue(Intent intent) {
        Context ctx = SampleApplication.getAppContext();
        LocationLibPreference llPreference = LocationLibPreference.getInstance();

        //呼び出し先のコンポーネント名を設定
        setIntentLocationLibComponent(intent);

        //Intentの種類（Operation）：測位開始
        intent.putExtra(LocationLibConst.KEY_OPERATION, LocationLibConst.OPERATION_START);

        //プロバイダー：GPS
        boolean gps = llPreference.getBoolean(ctx.getString(R.string.prf_lib_key_provider_gps), true);
        //プロバイダー：ネットワーク
        boolean network = llPreference.getBoolean(ctx.getString(R.string.prf_lib_key_provider_network), true);
        //プロバイダー種別（Provider）を配列型で格納する
        String[] provider = getProviderSetting(gps, network);
        if (provider != null) {
            intent.putExtra(LocationLibConst.KEY_PROVIDER, provider);
        }

        //測位間隔（Interval）
        String value = llPreference.getString(ctx.getString(R.string.prf_lib_key_interval), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_INTERVAL, Integer.valueOf(value));
        }

        //測位精度（AccuracyThreshold）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_accuracy), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_ACCURACY_THRESHOLD, Float.valueOf(value));
        }

        //測位結果通知アプリのパッケージ名（PackageName and ClassName）
        setIntentNotifyPackageAndClassName(intent);

        //デバッグモード（Debug）
        boolean bValue =  llPreference.getBoolean(ctx.getString(R.string.prf_lib_key_debug), true);
        intent.putExtra(LocationLibConst.KEY_DEBUG, bValue);

        //同一位置判定距離（SamePositionDeterminingDistance）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_same_position_determining_distance), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_SAME_POSITION_DETERMINING_DISTANCE, Integer.valueOf(value));
        }

        //同一位置判定回数（SamePositionDeterminingCountThreshold）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_same_position_determining_count), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_SAME_POSITION_DETERMINING_COUNT, Integer.valueOf(value));
        }

        //NotificationID
        intent.putExtra(LocationLibConst.KEY_NOTIFICATION_ID, 1);

        //Notification
        intent.putExtra(LocationLibConst.KEY_NOTIFICATION, createNotification());

        //APIキー（ApiKey）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_api_key), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_API_KEY, value);
        }

        //クライアントID（ClientID）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_client_id), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_CLIENT_ID, value);
        }

        //秘密鍵（Secret）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_secret), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_SECRET, value);
        }

        //認証サーバ環境 (AuthEnv)
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_auth_env), "0");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_AUTH_ENV, Integer.valueOf(value));
        }
    }

    /**
     * 引数のIntentに「測位停止」パラメータを設定する
     *
     * @param intent パラメータ設定先のIntent
     */
    public static void setIntentStopValue(Intent intent) {

        //呼び出し先のコンポーネント名を設定
        setIntentLocationLibComponent(intent);

        //Intentの種類（Operation）: 測位停止
        intent.putExtra(LocationLibConst.KEY_OPERATION, LocationLibConst.OPERATION_STOP);
    }

    /**
     * 引数のIntentに「現在地取得（単測位）」パラメータを設定する
     *
     * @param intent パラメータ設定先のIntent
     */
    public static void setIntentGetLocationValue(Intent intent) {

        Context ctx = SampleApplication.getAppContext();
        LocationLibPreference llPreference = LocationLibPreference.getInstance();

        //呼び出し先のコンポーネント名を設定
        setIntentLocationLibComponent(intent);

        //Intentの種類（Operation）：現在地取得（単測位）
        intent.putExtra(LocationLibConst.KEY_OPERATION, LocationLibConst.OPERATION_GET_LOCATION);

        //プロバイダー：GPS
        boolean gps = llPreference.getBoolean(ctx.getString(R.string.prf_lib_key_provider_gps), true);
        //プロバイダー：ネットワーク
        boolean network = llPreference.getBoolean(ctx.getString(R.string.prf_lib_key_provider_network), true);
        //プロバイダー種別（Provider）を配列型で格納する
        String[] provider = getProviderSetting(gps, network);
        if (provider != null) {
            intent.putExtra(LocationLibConst.KEY_PROVIDER, provider);
        }

        //測位結果通知アプリのパッケージ名（PackageName and ClassName）
        setIntentNotifyPackageAndClassName(intent);

        //デバッグモード（Debug）
        boolean bValue =  llPreference.getBoolean(ctx.getString(R.string.prf_lib_key_debug), true);
        intent.putExtra(LocationLibConst.KEY_DEBUG, bValue);

        //APIキー（ApiKey）
        String value = llPreference.getString(ctx.getString(R.string.prf_lib_key_api_key), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_API_KEY, value);
        }

        //クライアントID（ClientID）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_client_id), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_CLIENT_ID, value);
        }

        //秘密鍵（Secret）
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_secret), "");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_SECRET, value);
        }

        //認証サーバ環境 (AuthEnv)
        value = llPreference.getString(ctx.getString(R.string.prf_lib_key_auth_env), "0");
        if (!TextUtils.isEmpty(value)) {
            intent.putExtra(LocationLibConst.KEY_AUTH_ENV, Integer.valueOf(value));
        }
    }

    /**
     * 引数のIntentに「バージョン取得」パラメータを設定する
     *
     * @param intent パラメータ設定先のIntent
     */
    public static void setIntentGetVersionValue(Intent intent) {

        //呼び出し先のコンポーネント名を設定
        setIntentLocationLibComponent(intent);

        //Intentの種類（Operation）：バージョン取得
        intent.putExtra(LocationLibConst.KEY_OPERATION, LocationLibConst.OPERATION_GET_VERSION);

        //バージョン通知先のパッケージ名とクラス名（PackageName and ClassName）
        setIntentNotifyPackageAndClassName(intent);
    }

    /**
     * 引数のIntentに「動作値/設定値取得」パラメータを設定する
     *
     * @param intent パラメータ設定先のIntent
     * @param isOperationValue true：動作値の取得 ／ false：設定値の取得
     */
    public static void setIntentGetLibrarySettingsValue(Intent intent, boolean isOperationValue) {

        //呼び出し先のコンポーネント名を設定
        setIntentLocationLibComponent(intent);

        //Intentの種類（Operation）
        intent.putExtra(LocationLibConst.KEY_OPERATION, LocationLibConst.OPERATION_GET_LIBRARY_SETTINGS);

        //取得する設定情報種別（OperationValue）
        intent.putExtra(LocationLibConst.KEY_OPERATION_VALUE, isOperationValue);

        //設定値通知先のパッケージ名とクラス名（PackageName and ClassName）
        setIntentNotifyPackageAndClassName(intent);
    }

    /**
     * 引数のIntentに呼び出し先Component名として、「位置測位ライブラリ」サービスを設定する
     *
     * @param intent 設定追加先のIntent
     */
    private static void setIntentLocationLibComponent(Intent intent) {
        String packageName = SampleApplication.getAppContext().getPackageName();
        intent.setComponent(new ComponentName(packageName,
                "net.zenrindatacom.location.request.service.LocationRequestService"));
    }

    /**
     * 引数のIntentに「位置測位ライブラリからの通知を受けるパッケージ名とクラス名」を設定する
     * 本サンプルではLocationLibReceiveServiceで受け取る
     *
     * @param intent パラメータ設定先のIntent
     */
    private static void setIntentNotifyPackageAndClassName(Intent intent) {
        Context context = SampleApplication.getAppContext();
        //パッケージ名（PackageName)
        intent.putExtra(LocationLibConst.KEY_PACKAGE_NAME, context.getPackageName());

        //クラス名（ClassName）
        intent.putExtra(LocationLibConst.KEY_CLASS_NAME, LocationLibReceiveService.class.getName());
    }

    /**
     * 引数「プロバイダー」設定を配列に変換して返す
     *
     * @param isGPS プロバイダー（GPS）のON、OFF
     * @param isNetwork プロバイダー（NETWORK）のON、OFF
     * @return Providerの文字列配列
     */
    private static String[] getProviderSetting(boolean isGPS, boolean isNetwork) {

        ArrayList<String> provider = new ArrayList<>();
        if (isGPS) {
            provider.add("GPS");
        }
        if (isNetwork) {
            provider.add("NETWORK");
        }

        return provider.size()==0 ? null : provider.toArray(new String[0]);
    }

    /**
     * 測位開始後Android端末に表示するNotificationを作成する
     *
     * @return 作成したNotification
     */
    private static Notification createNotification() {
        Context ctx = SampleApplication.getAppContext();
        Notification notification ;
        //タイトル
        String title = ctx.getString(R.string.service_message_title);
        //本文
        String text = ctx.getString(R.string.service_message_body);
        //アイコン
        int icon = R.mipmap.ic_launcher;

        NotificationManager manager = (NotificationManager) ctx.getSystemService(Context.NOTIFICATION_SERVICE);
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            //Android8以降はNotification channelの設定が必要
            NotificationChannel channel = new NotificationChannel(
                    // アプリでユニークな ID
                    "app_channel",
                    // ユーザが「設定」アプリで見ることになるチャンネル名
                    ctx.getString(R.string.service_channel_name),
                    // チャンネルの重要度（0 ~ 4）
                    NotificationManager.IMPORTANCE_LOW
            );
            manager.createNotificationChannel(channel);

            notification = new Notification.Builder(ctx, "app_channel")
                    .setContentTitle(title)
                    .setContentText(text)
                    .setSmallIcon(icon)
                    .setAutoCancel(true)
                    .build();
        } else {
            //Android8未満
            notification = new Notification.Builder(ctx)
                    .setContentTitle(title)
                    .setContentText(text)
                    .setSmallIcon(icon)
                    .setAutoCancel(true)
                    .build();
        }

        return notification;
    }
}
